//---------------------------------------------------------------------------
// TDcjMessageDlg Component
// Copyright (c) 1998 Michael Trier.  All Rights Reserved.
// See source file for restrictions.
//---------------------------------------------------------------------------
#ifndef DcjMsgDgH
#define DcjMsgDgH
//---------------------------------------------------------------------------
#include "Dcjdef.h"
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
//---------------------------------------------------------------------------
// typedefs
typedef int TButtonID;
//---------------------------------------------------------------------------
// forward declarations
class PACKAGE TDcjMessageDlg;
class TDcjButtons;
class TDcjButton;
//---------------------------------------------------------------------------
// TDcjMessageForm Declaration
//---------------------------------------------------------------------------
class TDcjMessageForm : public Forms::TForm
{
private:
  TButtonID FButtonID;

public:
	/* TForm.Create */ __fastcall virtual TDcjMessageForm(
      Classes::TComponent* AOwner) : Forms::TForm(AOwner) { }
	/* TForm.CreateNew */ __fastcall TDcjMessageForm(Classes::TComponent* AOwner,
      int Dummy) : Forms::TForm(AOwner, Dummy) { }
	/* TForm.Destroy */ __fastcall virtual ~TDcjMessageForm(void) { }

  /* TWinControl.CreateParented */ __fastcall TDcjMessageForm(HWND ParentWindow)
      : Forms::TForm(ParentWindow) { }

  void __fastcall HelpButtonClick(System::TObject* Sender);
  void __fastcall ButtonClick(System::TObject* Sender);

  __property TButtonID ButtonID = {read=FButtonID};
};
//---------------------------------------------------------------------------
// Enums and Consts to support the button defaults
//---------------------------------------------------------------------------
// button alignment settings
enum TDcjHAlign { haLeft, haCenter, haRight };
enum TDcjVAlign { vaTop, vaCenter, vaBottom };

// constants
const int DEFAULT_BTN_WIDTH = 50;
const int DEFAULT_BTN_HEIGHT = 14;
const int DEFAULT_BTN_SPACING = 4;
const TDcjHAlign DEFAULT_BTN_HORIZ = haCenter;
const TDcjVAlign DEFAULT_BTN_VERT = vaCenter;
const TModalResult DEFAULT_BTN_MODALRESULT = mrNone;
//---------------------------------------------------------------------------
// TDcjButtons Declaration
//---------------------------------------------------------------------------
class TDcjButtons : public TCollection
{
private:
  // private data members
  TPersistent *FOwner;

protected:
  // overriden protected member functions
  TDcjButton* __fastcall GetItem(int Index);
  void __fastcall SetItem(int Index, TDcjButton* Value);
  DYNAMIC TPersistent* __fastcall GetOwner(void);

public:
  // constructor
  __fastcall TDcjButtons(TPersistent *Owner);

  // overriden public member functions
  virtual void __fastcall Assign(TPersistent* Source);
  TDcjButton* __fastcall Add(void);
  TDcjButton* __fastcall Add(AnsiString sCaption,
      TModalResult tResult = DEFAULT_BTN_MODALRESULT,
      int iWidth = DEFAULT_BTN_WIDTH,
      int iHeight = DEFAULT_BTN_HEIGHT);

  __property TDcjButton* Items[int Index] = {read=GetItem, write=SetItem};


  // public member functions
  void __fastcall DeleteItem(int ItemIndex);
  void __fastcall DeleteItem(TDcjButton *Item);

__published:
  // published properties
};
//---------------------------------------------------------------------------
// TDcjButton Declaration
//---------------------------------------------------------------------------
class TDcjButton : public TCollectionItem
{
private:
  // private class data members
  TDcjButtons *FCollection;
  TButtonID FButtonID;
  AnsiString FCaption;
  int FWidth;
  int FHeight;
  bool FDefault;
  bool FCancel;
  bool FHelp;
  TModalResult FModalResult;

  // private overriden helper member functions
  void __fastcall SetCollection(TDcjButtons* Value);
  TDcjButtons* __fastcall GetCollection(void);

protected:
  // protected class helper member functions
  void __fastcall SetButtonID(TButtonID Value);
  void __fastcall SetWidth(int Value);
  void __fastcall SetHeight(int Value);
  void __fastcall SetDefault(bool Value);
  void __fastcall SetCancel(bool Value);
  void __fastcall SetHelp(bool Value);
  void __fastcall SetModalResult(TModalResult Value);

public:
  // constructor
  __fastcall TDcjButton(TCollection *Collection);

  // overriden public member functions
  virtual void __fastcall Assign(TPersistent *Source);
  // overriden public properties
	__property TDcjButtons* Collection = {read=GetCollection, write=SetCollection,
      nodefault};

__published:
  // published class properties
  __property TButtonID ButtonID = {read=FButtonID, write=SetButtonID,
      default=0};
  __property AnsiString Caption = {read=FCaption, write=FCaption};
  __property int Width = {read=FWidth, write=SetWidth,
      default=DEFAULT_BTN_WIDTH};
  __property int Height = {read=FHeight, write=SetHeight,
      default=DEFAULT_BTN_HEIGHT};
  __property bool Default = {read=FDefault, write=SetDefault, default=false};
  __property bool Cancel = {read=FCancel, write=SetCancel, default=false};
  __property bool Help = {read=FHelp, write=SetHelp, default=false};
  __property TModalResult ModalResult = {read=FModalResult, write=SetModalResult,
      default=DEFAULT_BTN_MODALRESULT};
};
//---------------------------------------------------------------------------
// TDcjButtonContainer Declaration
//---------------------------------------------------------------------------
class TDcjButtonContainer : public TPersistent
{
private:
  // private data members
  TComponent *FOwner;
  TDcjHAlign FHorizontal;
  TDcjVAlign FVertical;
  int FSpacing;
  TDcjButtons *FButtons;

protected:
  // overriden protected member functions
  DYNAMIC TPersistent* __fastcall GetOwner(void);

public:
  // constructor / destructor
  __fastcall TDcjButtonContainer(TComponent *Owner);
  __fastcall ~TDcjButtonContainer(void);

  // overriden public member functions
  virtual void __fastcall Assign(TPersistent *Source);

__published:
  // published properties
  __property TDcjButtons *Buttons = {read=FButtons, write=FButtons};

  __property TDcjHAlign Horizontal = {read=FHorizontal, write=FHorizontal,
      default=DEFAULT_BTN_HORIZ};
  __property TDcjVAlign Vertical = {read=FVertical, write=FVertical,
      default=DEFAULT_BTN_VERT};
  __property int Spacing = {read=FSpacing, write=FSpacing,
      default=DEFAULT_BTN_SPACING};
};
//---------------------------------------------------------------------------
// TDcjMessageDlg Declaration
//---------------------------------------------------------------------------
class PACKAGE TDcjMessageDlg : public TComponent
{
private:
  AnsiString FCaption;
  TFont *FFont;
  AnsiString FMessage;
  TMsgDlgType FDialogType;
  TDcjButtonContainer *FButtonContainer;
  TModalResult FModalResult;
  TButtonID FButtonID;
  THelpContext FHelpContext;

protected:
  // protected member helper functions
  TDcjButtons* __fastcall GetButtons(void);
  TDcjMessageForm* __fastcall CreateMessageDialog(void);
  void __fastcall SetFont(TFont *Value);

public:
  __fastcall TDcjMessageDlg(TComponent* Owner);
  __fastcall ~TDcjMessageDlg(void);

  // overriden public member functions
  virtual void __fastcall Assign(TPersistent *Source);

  // public member functions
  bool __fastcall Execute(void);
  //* TODO - CREATE EXECUTE OVERLOAD MEMBER FUNCTION THAT SPECIFIES IF THE
  // DIALOG SHOULD BE MODAL OR MODELESS //

  __property TDcjButtons *Buttons = {read=GetButtons};
 	__property TModalResult ModalResult = {read=FModalResult};
  __property TButtonID ButtonID = {read=FButtonID};

__published:
  __property Name;
  __property Tag;

  __property AnsiString Caption = {read=FCaption, write=FCaption};
  __property TFont *Font = {read=FFont, write=SetFont};
  __property AnsiString Message = {read=FMessage, write=FMessage};
  __property TDcjButtonContainer *ButtonContainer = {read=FButtonContainer,
      write=FButtonContainer};
  __property TMsgDlgType DialogType = {read=FDialogType, write=FDialogType,
      default=mtWarning};
  __property THelpContext HelpContext = {read=FHelpContext,
      write=FHelpContext, default=0};

};
//---------------------------------------------------------------------------
#endif
